unit SVGCanvas;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils;

type
  TSVGTextAlignment = (svgLeft, svgCenter, svgRight);

  TSVGCanvas = class
  private
    FOutput: TStrings;
    FFormatSettings: TFormatSettings;
    FFillColor, FLineColor: String;
    FLineWidth: Double;
    FFontName: String;
    FFontSize: Double;
    FFontColor: String;
    FTextAlignment: TSVGTextAlignment;
    FIndent: String;
    FWriting: Boolean;
    procedure Write(AText: String);
    procedure WriteFmt(AText: String; const AParams: array of const);

    function GetFillStyleStr: String;
    function GetLineStyleStr: String;

  public
    constructor Create;
    destructor Destroy; override;
    procedure BeginDoc(AWidth, AHeight: Double);
    procedure EndDoc;

    procedure BeginGroup(ATitle: String);
    procedure EndGroup;

    procedure Rectangle(ALeft, ATop, AWidth, AHeight: Double);
    procedure Circle(ACenterX, ACenterY, ARadius: Double);
    procedure Ellipse(ACenterX, ACenterY, ARadiusX, ARadiusY: Double);
    procedure Line(AFromX, AFromY, AToX, AToY: Double);
    procedure SetFont(AFontName, AFontColor: String; AFontSize: Double);
    procedure TextOut(X, Y: Double; AText: String);

    procedure SaveToFile(AFileName: String);
    procedure SaveToStream(AStream: TStream);

    property FillColor: String read FFillColor write FFillColor;
    property LineColor: String read FLineColor write FLineColor;
    property LineWidth: Double read FLineWidth write FLineWidth;
    property FontName: String read FFontName write FFontName;
    property FontColor: String read FFontColor write FFontColor;
    property FontSize: Double read FFontSize write FFontSize;
    property TextAlignment: TSVGTextAlignment read FTextAlignment write FTextAlignment;
  end;


implementation

constructor TSVGCanvas.Create;
begin
  inherited;
  FOutput := TStringList.Create;
  FFormatSettings := DefaultFormatSettings;
  FFormatSettings.DecimalSeparator := '.';   // Schreibe Dezimalzahlen mit Punkt
  FFillColor := 'none';
  FLineColor := 'none';
  FLineWidth := 1;
  FFontName := 'Courier';
  FFontcolor := 'black';
  FFontSize := 36;
end;

destructor TSVGCanvas.Destroy;
begin
  FOutput.Free;
  inherited;
end;

procedure TSVGCanvas.BeginDoc(AWidth, AHeight: Double);
begin
  FWriting := true;
  WriteFmt(
    '<svg xmlns="http://www.w3.org/2000/svg" width="%g" height="%g" >' ,
    [AWidth, AHeight]
  );
  FIndent := '  ';
end;

procedure TSVGCanvas.EndDoc;
begin
  FIndent := '';
  Write('</svg>');
  FWriting := false;
end;

procedure TSVGCanvas.BeginGroup(ATitle: String);
begin
  WriteFmt('%s<g>', [FIndent]);
  FIndent := FIndent + '  ';
  WriteFmt('%s<title>%s</title>', [FIndent, ATitle]);
end;

procedure TSVGCanvas.EndGroup;
begin
  SetLength(FIndent, Length(FIndent) - 2);
  WriteFmt('%s</g>', [FIndent]);
end;

procedure TSVGCanvas.SaveToFile(AFileName: String);
begin
  if FWriting then
    EndDoc;
  FOutput.SaveToFile(AFileName);
end;

procedure TSVGCanvas.SaveToStream(AStream: TStream);
begin
  if FWriting then
    EndDoc;
  FOutput.SaveToStream(AStream);
end;

procedure TSVGCanvas.SetFont(AFontName, AFontColor: String; AFontSize: Double);
begin
  FFontName := AFontName;
  FFontSize := AFontSize;
  FFontColor := AFontColor;
end;

procedure TSVGCanvas.Write(AText: String);
begin
  if not FWriting then
    raise Exception.Create('Writing allowed only between BeginDoc and EndDoc');
  FOutput.Add(AText);
end;

procedure TSVGCanvas.WriteFmt(AText: String; const AParams: array of const);
begin
  Write(Format(AText, AParams, FFormatSettings));
end;

function TSVGCanvas.GetFillStyleStr: String;
begin
  Result := Format('fill="%s" ', [FFillColor]);
end;

function TSVGCanvas.GetLineStyleStr: String;
begin
  Result := Format('stroke="%s" stroke-width="%g" ', [FLineColor, FLineWidth]);
end;

procedure TSVGCanvas.Circle(ACenterX, ACenterY, ARadius: Double);
begin
  WriteFmt('%s<circle cx="%g" cy="%g" r="%g" %s%s/>',
    [FIndent, ACenterX, ACenterY, ARadius, GetFillStyleStr, GetLineStyleStr]
  );
end;

procedure TSVGCanvas.Ellipse(ACenterX, ACenterY, ARadiusX, ARadiusY: Double);
begin
  WriteFmt('%s<ellipse cx="%g" cy="%g" rx="%g" ry="%g" %s%s/>',
    [FIndent, ACenterX, ACenterY, ARadiusX, ARadiusY, GetFillStyleStr, GetlineStyleStr]
  );
end;

procedure TSVGCanvas.Line(AFromX, AFromY, AToX, AToY: Double);
begin
  WriteFmt('%s<line x1="%g" y1="%g" x2="%g" y2="%g" %s/>',
    [FIndent, AFromX, AFromY, AToX, AToY, GetLinestyleStr]
  );
end;

procedure TSVGCanvas.Rectangle(ALeft, ATop, AWidth, AHeight: Double);
begin
  WriteFmt('%s<rect x="%g" y="%g" width="%g" height="%g" %s%s/>',
    [FIndent, ALeft, ATop, AWidth, AHeight, GetFillStyleStr, GetLineStyleStr]
  );
end;

procedure TSVGCanvas.TextOut(X, Y: Double; AText: String);
var
  align: String;
begin
  case FTextAlignment of
    svgLeft: align := '';
    svgCenter: align := ' text-anchor="middle"';
    svgRight: align := ' text-anchor="end"';
  end;

  WriteFmt(
    '%s<text x="%g" y="%g" fill="%s" font-name="%s" font-size="%g"%s>'+AText+'</text>',
    [FIndent, X, Y, FFontColor, FFontName, FFontSize, align]
  );
end;


end.

